
/**
 * PM 2015
 * Ana Aldescu
 * Razvan Ivanoiu
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>

#include "utils.h"
#include "lcd.h"

int main()
{
	int i, j, rc_x, rc_y;
	int turn = 0;
	char buf[1], c;

	USART0_init(103);

	/* IO init */
	io_init();

	/* LCD init */
	lcd_init();
	lcd_contrast(0x40);

	/* Control matrix init */
	init_matrix();

	/* Start game */
	print_hello();

	/* Place airplane */
	while (1) {
		if (!(PINB & (1 << PB4))) {
			lcd_clear();
			_delay_ms(2000);

			print_grid();
			draw_cursor(7 + c_x * 8, c_y + 2);
		}

		if (!(PINB & (1 << PB5))) {
			if ((c_y == 1 || c_y == 2) && (c_x >= 2 && c_x <= 6)) {
				put_airplane(7 + c_x * 8, c_y + 2);
				mark_plane1(c_x, c_y);
				p1_x = c_x;
				p1_y = c_y;

				_delay_ms(20000);
				break;
			}
		}

		if (!(PINB & (1 << PB0))) {
			action(PB0, turn);
		}

		if (!(PINB & (1 << PB1))) {
			action(PB1, turn);
		}

		if (!(PINB & (1 << PB3))) {
			action(PB3, turn);
		}

		if (!(PINB & (1 << PB2))) {
			action(PB2, turn);
		}

		_delay_ms(2000);
	}

	/* Send plane coords to opponent */
	itoa(c_x, buf, 10);
	USART0_print(buf);
	itoa(c_y, buf, 10);
	USART0_print(buf);

	/* Recv opponent's plane coords */
	c = USART0_receive();
	rc_x = c - '0';
	c = USART0_receive();
	rc_y = c - '0';
	mark_plane2(rc_x, rc_y);
	p2_x = rc_x;
	p2_y = rc_y;

	turn = 1;

	/* Game communication */
	while (1) {
		// Hit slave plane si send pe seriala
		if (turn) {
			lcd_clear();
			print_turn();
			_delay_ms(20000);

			lcd_clear();
			print_grid();

			for (i = 0; i < X_SIZE; i++) {
				for (j = 0; j < Y_SIZE; j++) {
					if (matrix_2[i][j] == HIT) {
						draw_hit(7 + i * 8, j + 2);
					}

					if (matrix_2[i][j] == MISS) {
						draw_miss(7 + i * 8, j + 2);
					}
				}
			}

			c_x = CENTER_X;
			c_y = CENTER_Y;
			draw_cursor(7 + c_x * 8, c_y + 2);

			while (1) {
				if (!(PINB & (1 << PB4))) {
					if (matrix_2[c_x][c_y] == PLANE) {
						draw_hit(7 + c_x * 8, c_y + 2);
						matrix_2[c_x][c_y] = HIT;
					} else {
						draw_miss(7 + c_x * 8, c_y + 2);
						matrix_2[c_x][c_y] = MISS;
					}

					if (c_x == p2_x && c_y == (p2_y - 1)) {
						lcd_clear();
						print_win();

						USART0_print("l");
						itoa(c_y, buf, 10);
						USART0_print(buf);

						while(1);
					}

					/* Send hit coords to opponent */
					itoa(c_x, buf, 10);
					USART0_print(buf);
					itoa(c_y, buf, 10);
					USART0_print(buf);

					_delay_ms(20000);
					turn = 0;
					break;
				}

				if (!(PINB & (1 << PB0))) {
					action(PB0, turn);
				}

				if (!(PINB & (1 << PB1))) {
					action(PB1, turn);
				}

				if (!(PINB & (1 << PB3))) {
					action(PB3, turn);
				}

				if (!(PINB & (1 << PB2))) {
					action(PB2, turn);
				}

				_delay_ms(2000);
			}
		} else {
			lcd_clear();
			print_wait();
			_delay_ms(20000);

			lcd_clear();
			print_grid();

			for (i = 0; i < X_SIZE; i++) {
				for (j = 0; j < Y_SIZE; j++) {
					if (matrix_1[i][j] == HIT) {
						draw_hit(7 + i * 8, j + 2);
					}

					if (matrix_1[i][j] == MISS) {
						draw_miss(7 + i * 8, j + 2);
					}

					if (matrix_1[i][j] == PLANE) {
						draw_cursor(7 + i * 8, j + 2);
					}
				}
			}

			/* Recv opponent's hit coords */
			c = USART0_receive();

			if (c == 'l') {
				lcd_clear();
				print_lose();

				while(1);
			}

			rc_x = c - '0';
			c = USART0_receive();
			rc_y = c - '0';

			if (matrix_1[rc_x][rc_y] == PLANE) {
				draw_hit(7 + rc_x * 8, rc_y + 2);
				matrix_1[rc_x][rc_y] = HIT;
			} else {
				draw_miss(7 + rc_x * 8, rc_y + 2);
				matrix_1[rc_x][rc_y] = MISS;
			}

			_delay_ms(20000);
			turn = 1;
		}

		_delay_ms(2000);
	}

	return 0;
}

